import os

from openhtf.plugs.user_input import UserInput

from spintop_openhtf import TestPlan, conf
from spintop_openhtf.util.markdown import markdown

# This defines the name of the testbench.
plan = TestPlan('examples.getting_started', store_location='here')

HERE = os.path.abspath(os.path.dirname(__file__))

FORM_LAYOUT = {
    'schema':{
        'title': "First and Last Name",
        'type': "object",
        'required': ["firstname", "lastname"],
        'properties': {
            'firstname': {
                'type': "string", 
                'title': "First Name"
            },
            'lastname': {
                'type': "string", 
                'title': "Last Name"
            },
            'radio_choice': {
                'type': "string", 
                'default': "default",
                'title': "Radio Choice"
            },
            'checkbox': {
                'type': "boolean", 
                'default': False,
                'title': "Checkbox"
            },
            'dropdown': {
                'type': "string", 
                'default': "default",
                'title': "Dropdown"
            },
        }
    },
    'layout':[
        {
            "type": "help",
            "helpvalue": markdown("""
#Welcome to the Getting Started !

##Introduction

This test simply showcases the custom forms by asking information to the tester and then showing it to him.

##Image showcase

<img src="%s" width="200px" />

""" % plan.image_url(os.path.join(HERE, 'spinhub-app-icon.png'))
            )
        },
        "firstname",
        "lastname",
        {
            "key": "radio_choice",
            "type": "radiobuttons",
            "titleMap": [
                { "value": "One", "name": "One" },
                { "value": "Two", "name": "Two" }
            ]
        },
        "checkbox",
        {
            "key": "dropdown",
            "type": "select",
            "titleMap": [
                { "value": "One", "name": "One" },
                { "value": "Two", "name": "Two" }
            ]
        }
    ]
}

"""Custom Plugs"""

class GreetPlug(UserInput):
    def prompt_tester_information(self):
        self.__response = self.prompt_form(FORM_LAYOUT)
        return self.__response
    
    def greet_tester(self):
        try:
            self.prompt("""
Hello {firstname} {lastname} ! 

You chose "{radio_choice}" from the radio buttons.
You chose "{dropdown}" from the dropdown menu.

""".format(**self.__response)
            )
        except AttributeError:
            raise Exception("Cannot greet tester before prompt_information")

""" Test Plan """

@plan.trigger('Hello-World')
@plan.plug(greet=GreetPlug)
def hello_world(test, greet):
    """Says Hello World !
    
# Hello World Test
    
Welcome to the **hello world** test.
    """
    response = greet.prompt_tester_information()
    test.dut_id = response['firstname']
    
@plan.testcase('Greet the tester')
@plan.plug(greet=GreetPlug)
def greet_tester(test, greet):
    greet.greet_tester()

if __name__ == '__main__':
    plan.run()


import os

from openhtf.plugs.user_input import UserInput
from openhtf import conf

from spintop_openhtf import TestPlan

""" Test Plan """

plan = TestPlan('examples.spintop_integration')

@plan.testcase('Hello-World')
@plan.plug(prompts=UserInput)
def hello_world(test, prompts):
    """Says Hello World!"""
    test.logger.info('Hello World')

if __name__ == '__main__':
    plan.enable_spintop()
    conf.load(spintop_org_id='tackv')
    plan.run()



""" This is the module-level summary of the hello_world test.
"""

import os
import time
import random

from spintop_openhtf import TestPlan, PhaseResult

import openhtf as htf

from openhtf.plugs.user_input import UserInput, PromptType, SecondaryOptionOccured
from openhtf.util import conf

""" Test Plan """

plan = TestPlan('examples.hello_world')
# plan.define_top_level_component('nets.yml')

HERE = os.path.abspath(os.path.dirname(__file__))

FORM_LAYOUT = {
    'schema':{
        'title': "Todo",
        'type': "object",
        'required': ["title"],
        'properties': {
            'title': {
                'type': "string", 
                'title': "Title"
            },
            'done': {'type': "boolean", 'title': "Done?", 'default': False}
        }
    },
    'layout':[
        "done",
        {
            'key': "title",
            'type': "section",
        }
    ]
}

spinsuite_image_url = plan.file_provider.create_url(os.path.join(HERE, 'spinhub-app-icon.png'))

@plan.testcase('Hello-World')
@htf.plugs.plug(prompts=UserInput)
@htf.PhaseOptions(requires_state=True)
def hello_world(state, prompts):
    """Says Hello World !
    
# Hello World Test
    
Welcome to the **hello world** test.
    """
    try:
        prompts.prompt_form(FORM_LAYOUT, prompt_type=PromptType.PASS_FAIL)
        prompts.prompt('Cancel', prompt_type=PromptType.OKAY_CANCEL)
        prompts.prompt('PASS', prompt_type=PromptType.PASS_FAIL)
    except SecondaryOptionOccured:
        return PhaseResult.FAIL_AND_CONTINUE 
        
    state.logger.info('Hello World')
    prompts.prompt("""
# {url}

This is **Awesome**

Please tell me more.
![My-Image]({url})
                    """.format(url=spinsuite_image_url))


@plan.testcase('Hello-World-2')
@htf.plugs.plug(prompts=UserInput)
def hello_world(test, prompts):
    """Says Hello World 2 !"""
    test.logger.info('Hello World 2')
    test.state['skip'] = True

sub_group = plan.sub_sequence('sub-group')

@sub_group.testcase('sub Hello', run_if=lambda state: not state.get('skip', False))
def hello_world(test):
    """Says Sub hello."""
    test.logger.info('Sub hello')

@sub_group.teardown('sub cleanup')
def hello_world(test):
    """Says Sub cleanup."""
    test.logger.info('Sub cleanup')
    
@plan.teardown('cleanup')
def cleanup(test):
    """Says Cleaned up."""
    test.logger.info('Cleaned up.')

if __name__ == '__main__':
    plan.run()







from .base import UnboundPlug

import typing
from functools import wraps
from collections.abc import Sequence

import telnetlib


class TelnetError(Exception):
    pass


def _telnet_client_connected(function):
    @wraps(function)
    def check_connected(*args, **kwargs):
        _self = args[0]
        if not _self.is_connected():
            _self.logger.info("Connection is no longer alive")
            _self.open()
        return function(*args, **kwargs)

    return check_connected


class TelnetInterface(UnboundPlug):

    def __init__(self, addr, port=4343):
        super().__init__()
        self.tn = None
        self.addr = addr
        self.port = port

    def open(self, _client=None):
        # _client allows to pass in a mock for testing
        self.logger.info("(Initiating Telnet connection at %s)", self.addr)
        self.logger.info("(addr={}:{})".format(self.addr, self.port))
        try:
            if _client is None:
                _client = telnetlib.Telnet(self.addr, self.port)
            self.tn = _client
            self.tn.open(self.addr, port=self.port)
        except Exception as e:
            raise TelnetError("Unable to connect to Telnet host: " + str(e))

    def close(self):
        try:
            self.logger.info("Closing Telnet connection")
            self.tn.close()
        except:
            pass

    def is_connected(self):
        try:
            self.tn.write("\r\n".encode())
            # If the Telnet object is not connected, an AttributeError is raised
        except AttributeError:
            return False
        else:
            return True

    @_telnet_client_connected
    def execute_command(self, command: str, timeout: float = 10, targets=None):
        """Send a :obj:`command` and wait for it to execute.

        Args:
            command (str): The command to send. End of lines are automatically managed. For example execute_command('ls')
            will executed the ls command.
            timeout (float, optional): The timeout in second to wait for the command to finish executing. Defaults to 10.
            targets ([type], optional): [description]. Defaults to None.

        Returns:
            str: output response from the Telnet client

        Raises:
            TelnetError:
                Raised when reading the output goes wrong.
        """

        output = ""
        if command != "":
            from time import sleep
            self.logger.info("(Timeout %.1fs)" % (timeout))
            self.tn.write((command).encode() + b"\r\n")
            self.logger.debug("> {!r}".format(command))

            try:
                output = self.tn.read_very_eager()
                output = output.decode()
            except Exception as e:
                raise TelnetError(e)
        else:
            pass

        return output



import os

from openhtf.plugs.user_input import UserInput

from spintop_openhtf import TestPlan, conf
from spintop_openhtf.util.markdown import markdown

# This defines the name of the testbench.
plan = TestPlan('empty')

@plan.testcase('Nothing')
def test_something(test):
    pass

if __name__ == '__main__':
    plan.run()


from functools import lru_cache

from openhtf import conf

from spintop import Spintop

from spintop_openhtf.transforms.openhtf_fmt import OpenHTFTestRecordTransformer

conf.declare(
    'spintop_api_url', 
    description='The spintop API URL to use. Overrides the default value from env (SPINTOP_API_URI).', 
    default_value=None
)

conf.declare(
    'spintop_org_id', 
    description='The spintop organization which will receive test results.', 
    default_value=None
)

@lru_cache()
def get_spintop(spintop=None):
    if spintop is None:
        spintop = Spintop(api_url=conf['spintop_api_url'], org_id=conf['spintop_org_id'])
    return spintop

def spintop_callback_factory(spintop=None):
    transform = OpenHTFTestRecordTransformer()
    def spintop_callback(test):
        record = transform(test)
        get_spintop(spintop).records.update([record])
    
    return spintop_callback